@@ -8,16 +8,16 @@ import (
 	"encoding/gob"
 	"encoding/json"
 	"math"
-	"reflect"
 	"strings"
 	"testing"
 	"time"
 
-	msgpack2 "github.com/ugorji/go-msgpack"
-	"github.com/ugorji/go/codec"
+	gomsgpack "github.com/ugorji/go-msgpack"
+	gocodec "github.com/ugorji/go/codec"
 	. "gopkg.in/check.v1"
 
 	"gopkg.in/vmihailenco/msgpack.v2"
+	"gopkg.in/vmihailenco/msgpack.v2/codes"
 )
 
 type nameStruct struct {
@@ -40,33 +40,27 @@ func (t *MsgpackTest) SetUpTest(c *C) {
 	t.dec = msgpack.NewDecoder(bufio.NewReader(t.buf))
 }
 
-func (t *MsgpackTest) TestUint(c *C) {
+func (t *MsgpackTest) TestUint64(c *C) {
 	table := []struct {
-		v uint
+		v uint64
 		b []byte
 	}{
 		{0, []byte{0x00}},
 		{1, []byte{0x01}},
-		{2, []byte{0x02}},
-		{125, []byte{0x7d}},
-		{126, []byte{0x7e}},
-		{127, []byte{0x7f}},
-		{128, []byte{0xcc, 0x80}},
-		{253, []byte{0xcc, 0xfd}},
-		{254, []byte{0xcc, 0xfe}},
-		{255, []byte{0xcc, 0xff}},
-		{256, []byte{0xcd, 0x01, 0x00}},
-		{65533, []byte{0xcd, 0xff, 0xfd}},
-		{65534, []byte{0xcd, 0xff, 0xfe}},
-		{65535, []byte{0xcd, 0xff, 0Xff}},
-		{65536, []byte{0xce, 0x00, 0x01, 0x00, 0x00}},
-		{4294967293, []byte{0xce, 0xff, 0xff, 0xff, 0xfd}},
-		{4294967294, []byte{0xce, 0xff, 0xff, 0xff, 0xfe}},
-		{4294967295, []byte{0xce, 0xff, 0xff, 0xff, 0xff}},
-		{4294967296, []byte{0xcf, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00}},
-		{18446744073709551613, []byte{0xcf, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfd}},
-		{18446744073709551614, []byte{0xcf, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe}},
-		{18446744073709551615, []byte{0xcf, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff}},
+		{math.MaxInt8 - 1, []byte{0x7e}},
+		{math.MaxInt8, []byte{0x7f}},
+		{math.MaxInt8 + 1, []byte{0xcc, 0x80}},
+		{math.MaxUint8 - 1, []byte{0xcc, 0xfe}},
+		{math.MaxUint8, []byte{0xcc, 0xff}},
+		{math.MaxUint8 + 1, []byte{0xcd, 0x1, 0x0}},
+		{math.MaxUint16 - 1, []byte{0xcd, 0xff, 0xfe}},
+		{math.MaxUint16, []byte{0xcd, 0xff, 0xff}},
+		{math.MaxUint16 + 1, []byte{0xce, 0x0, 0x1, 0x0, 0x0}},
+		{math.MaxUint32 - 1, []byte{0xce, 0xff, 0xff, 0xff, 0xfe}},
+		{math.MaxUint32, []byte{0xce, 0xff, 0xff, 0xff, 0xff}},
+		{math.MaxUint32 + 1, []byte{0xcf, 0x0, 0x0, 0x0, 0x1, 0x0, 0x0, 0x0, 0x0}},
+		{math.MaxInt64 - 1, []byte{0xcf, 0x7f, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe}},
+		{math.MaxInt64, []byte{0xcf, 0x7f, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff}},
 	}
 	for _, r := range table {
 		var int64v int64
@@ -88,63 +82,48 @@ func (t *MsgpackTest) TestUint(c *C) {
 		case int64:
 			c.Assert(iface, Equals, int64(r.v))
 		case uint64:
-			c.Assert(iface, Equals, uint64(r.v))
+			c.Assert(iface, Equals, r.v)
 		default:
 			panic("not reached")
 		}
 	}
 }
 
-func (t *MsgpackTest) TestInt(c *C) {
+func (t *MsgpackTest) TestInt64(c *C) {
 	table := []struct {
-		v int
+		v int64
 		b []byte
 	}{
-		{-9223372036854775808, []byte{0xd3, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}},
-		{-9223372036854775807, []byte{0xd3, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01}},
-		{-9223372036854775806, []byte{0xd3, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02}},
-		{-2147483651, []byte{0xd3, 0xff, 0xff, 0xff, 0xff, 0x7f, 0xff, 0xff, 0xfd}},
-		{-2147483650, []byte{0xd3, 0xff, 0xff, 0xff, 0xff, 0x7f, 0xff, 0xff, 0xfe}},
-		{-2147483649, []byte{0xd3, 0xff, 0xff, 0xff, 0xff, 0x7f, 0xff, 0xff, 0xff}},
-		{-2147483648, []byte{0xd2, 0x80, 0x00, 0x00, 0x00}},
-		{-2147483647, []byte{0xd2, 0x80, 0x00, 0x00, 0x01}},
-		{-2147483646, []byte{0xd2, 0x80, 0x00, 0x00, 0x002}},
-		{-32771, []byte{0xd2, 0xff, 0xff, 0x7f, 0xfd}},
-		{-32770, []byte{0xd2, 0xff, 0xff, 0x7f, 0xfe}},
-		{-32769, []byte{0xd2, 0xff, 0xff, 0x7f, 0xff}},
-		{-32768, []byte{0xd1, 0x80, 0x00}},
-		{-32767, []byte{0xd1, 0x80, 0x01}},
-		{-131, []byte{0xd1, 0xff, 0x7d}},
-		{-130, []byte{0xd1, 0xff, 0x7e}},
-		{-129, []byte{0xd1, 0xff, 0x7f}},
-		{-128, []byte{0xd0, 0x80}},
-		{-127, []byte{0xd0, 0x81}},
-		{-34, []byte{0xd0, 0xde}},
+		{math.MinInt64, []byte{0xd3, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}},
+		{math.MinInt32 - 1, []byte{0xd3, 0xff, 0xff, 0xff, 0xff, 0x7f, 0xff, 0xff, 0xff}},
+		{math.MinInt32, []byte{0xd2, 0x80, 0x00, 0x00, 0x00}},
+		{math.MinInt32 + 1, []byte{0xd2, 0x80, 0x00, 0x00, 0x01}},
+		{math.MinInt16 - 1, []byte{0xd2, 0xff, 0xff, 0x7f, 0xff}},
+		{math.MinInt16, []byte{0xd1, 0x80, 0x00}},
+		{math.MinInt16 + 1, []byte{0xd1, 0x80, 0x01}},
+		{math.MinInt8 - 1, []byte{0xd1, 0xff, 0x7f}},
+		{math.MinInt8, []byte{0xd0, 0x80}},
+		{math.MinInt8 + 1, []byte{0xd0, 0x81}},
 		{-33, []byte{0xd0, 0xdf}},
 		{-32, []byte{0xe0}},
 		{-31, []byte{0xe1}},
+		{-1, []byte{0xff}},
 		{0, []byte{0x00}},
 		{1, []byte{0x01}},
-		{126, []byte{0x7e}},
-		{127, []byte{0x7f}},
-		{128, []byte{0xd1, 0x00, 0x80}},
-		{129, []byte{0xd1, 0x00, 0x81}},
-		{130, []byte{0xd1, 0x00, 0x82}},
-		{32765, []byte{0xd1, 0x7f, 0xfd}},
-		{32766, []byte{0xd1, 0x7f, 0xfe}},
-		{32767, []byte{0xd1, 0x7f, 0xff}},
-		{32768, []byte{0xd2, 0x00, 0x00, 0x80, 0x00}},
-		{32769, []byte{0xd2, 0x00, 0x00, 0x80, 0x01}},
-		{32770, []byte{0xd2, 0x00, 0x00, 0x80, 0x02}},
-		{2147483645, []byte{0xd2, 0x7f, 0xff, 0xff, 0xfd}},
-		{2147483646, []byte{0xd2, 0x7f, 0xff, 0xff, 0xfe}},
-		{2147483647, []byte{0xd2, 0x7f, 0xff, 0xff, 0xff}},
-		{2147483648, []byte{0xd3, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00}},
-		{2147483649, []byte{0xd3, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x01}},
-		{2147483650, []byte{0xd3, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x02}},
-		{4294967296, []byte{0xd3, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00}},
-		{4294967297, []byte{0xd3, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x01}},
-		{4294967298, []byte{0xd3, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x02}},
+		{math.MaxInt8 - 1, []byte{0x7e}},
+		{math.MaxInt8, []byte{0x7f}},
+		{math.MaxInt8 + 1, []byte{0xcc, 0x80}},
+		{math.MaxUint8 - 1, []byte{0xcc, 0xfe}},
+		{math.MaxUint8, []byte{0xcc, 0xff}},
+		{math.MaxUint8 + 1, []byte{0xcd, 0x1, 0x0}},
+		{math.MaxUint16 - 1, []byte{0xcd, 0xff, 0xfe}},
+		{math.MaxUint16, []byte{0xcd, 0xff, 0xff}},
+		{math.MaxUint16 + 1, []byte{0xce, 0x0, 0x1, 0x0, 0x0}},
+		{math.MaxUint32 - 1, []byte{0xce, 0xff, 0xff, 0xff, 0xfe}},
+		{math.MaxUint32, []byte{0xce, 0xff, 0xff, 0xff, 0xff}},
+		{math.MaxUint32 + 1, []byte{0xcf, 0x0, 0x0, 0x0, 0x1, 0x0, 0x0, 0x0, 0x0}},
+		{math.MaxInt64 - 1, []byte{0xcf, 0x7f, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe}},
+		{math.MaxInt64, []byte{0xcf, 0x7f, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff}},
 	}
 	for _, r := range table {
 		var int64v int64
@@ -164,7 +143,7 @@ func (t *MsgpackTest) TestInt(c *C) {
 		c.Assert(err, IsNil)
 		switch iface.(type) {
 		case int64:
-			c.Assert(iface, Equals, int64(r.v))
+			c.Assert(iface, Equals, r.v)
 		case uint64:
 			c.Assert(iface, Equals, uint64(r.v))
 		default:
@@ -178,22 +157,30 @@ func (t *MsgpackTest) TestFloat32(c *C) {
 		v float32
 		b []byte
 	}{
-		{.1, []byte{0xca, 0x3d, 0xcc, 0xcc, 0xcd}},
-		{.2, []byte{0xca, 0x3e, 0x4c, 0xcc, 0xcd}},
-		{-.1, []byte{0xca, 0xbd, 0xcc, 0xcc, 0xcd}},
-		{-.2, []byte{0xca, 0xbe, 0x4c, 0xcc, 0xcd}},
-		{float32(math.Inf(1)), []byte{0xca, 0x7f, 0x80, 0x00, 0x00}},
-		{float32(math.Inf(-1)), []byte{0xca, 0xff, 0x80, 0x00, 0x00}},
-		{math.MaxFloat32, []byte{0xca, 0x7f, 0x7f, 0xff, 0xff}},
-		{math.SmallestNonzeroFloat32, []byte{0xca, 0x0, 0x0, 0x0, 0x1}},
+		{0.1, []byte{codes.Float, 0x3d, 0xcc, 0xcc, 0xcd}},
+		{0.2, []byte{codes.Float, 0x3e, 0x4c, 0xcc, 0xcd}},
+		{-0.1, []byte{codes.Float, 0xbd, 0xcc, 0xcc, 0xcd}},
+		{-0.2, []byte{codes.Float, 0xbe, 0x4c, 0xcc, 0xcd}},
+		{float32(math.Inf(1)), []byte{codes.Float, 0x7f, 0x80, 0x00, 0x00}},
+		{float32(math.Inf(-1)), []byte{codes.Float, 0xff, 0x80, 0x00, 0x00}},
+		{math.MaxFloat32, []byte{codes.Float, 0x7f, 0x7f, 0xff, 0xff}},
+		{math.SmallestNonzeroFloat32, []byte{codes.Float, 0x0, 0x0, 0x0, 0x1}},
 	}
 	for _, r := range table {
 		c.Assert(t.enc.Encode(r.v), IsNil)
 		c.Assert(t.buf.Bytes(), DeepEquals, r.b, Commentf("err encoding %v", r.v))
 
-		var v float32
-		c.Assert(t.dec.Decode(&v), IsNil)
-		c.Assert(v, Equals, r.v)
+		var f32 float32
+		c.Assert(t.dec.Decode(&f32), IsNil)
+		c.Assert(f32, Equals, r.v)
+
+		// Pass pointer to skip fast-path and trigger reflect.
+		c.Assert(t.enc.Encode(&r.v), IsNil)
+		c.Assert(t.buf.Bytes(), DeepEquals, r.b, Commentf("err encoding %v", r.v))
+
+		var f64 float64
+		c.Assert(t.dec.Decode(&f64), IsNil)
+		c.Assert(float32(f64), Equals, r.v)
 
 		c.Assert(t.enc.Encode(r.v), IsNil)
 		iface, err := t.dec.DecodeInterface()
@@ -485,37 +472,6 @@ func (t *MsgpackTest) TestStructNil(c *C) {
 	c.Assert(dst.Name, Equals, "foo")
 }
 
-type testStruct struct {
-	Name   string
-	Tm     time.Time
-	Data   []byte
-	Colors []string
-}
-
-func TestStruct(t *testing.T) {
-	in := testStruct{
-		Name:   "hello world",
-		Tm:     time.Now(),
-		Data:   []byte{1, 2, 3},
-		Colors: []string{"red", "orange", "yellow", "green", "blue", "violet"},
-	}
-	var out testStruct
-
-	b, err := msgpack.Marshal(in)
-	if err != nil {
-		t.Fatal(err)
-	}
-
-	err = msgpack.Unmarshal(b, &out)
-	if err != nil {
-		t.Fatal(err)
-	}
-
-	if !reflect.DeepEqual(out, in) {
-		t.Fatal("%#v != %#v", out, in)
-	}
-}
-
 func (t *MsgpackTest) TestStructUnknownField(c *C) {
 	in := struct {
 		Field1 string
@@ -541,6 +497,10 @@ type coderStruct struct {
 	name string
 }
 
+type wrapperStruct struct {
+	coderStruct `msgpack:",inline"`
+}
+
 var (
 	_ msgpack.CustomEncoder = &coderStruct{}
 	_ msgpack.CustomDecoder = &coderStruct{}
@@ -592,6 +552,14 @@ func (t *MsgpackTest) TestPtrToCoder(c *C) {
 	c.Assert(out.Name(), Equals, "hello")
 }
 
+func (t *MsgpackTest) TestWrappedCoder(c *C) {
+	in := &wrapperStruct{coderStruct: coderStruct{name: "hello"}}
+	var out wrapperStruct
+	c.Assert(t.enc.Encode(in), IsNil)
+	c.Assert(t.dec.Decode(&out), IsNil)
+	c.Assert(out.Name(), Equals, "hello")
+}
+
 //------------------------------------------------------------------------------
 
 type struct2 struct {
@@ -779,10 +747,10 @@ func BenchmarkIntBinary(b *testing.B) {
 	}
 }
 
-func BenchmarkIntMsgpack2(b *testing.B) {
+func BenchmarkIntUgorjiGoMsgpack(b *testing.B) {
 	buf := &bytes.Buffer{}
-	dec := msgpack2.NewDecoder(buf, nil)
-	enc := msgpack2.NewEncoder(buf)
+	dec := gomsgpack.NewDecoder(buf, nil)
+	enc := gomsgpack.NewEncoder(buf)
 
 	var out int
 	for i := 0; i < b.N; i++ {
@@ -795,10 +763,10 @@ func BenchmarkIntMsgpack2(b *testing.B) {
 	}
 }
 
-func BenchmarkIntMsgpack3(b *testing.B) {
+func BenchmarkIntUgorjiGoCodec(b *testing.B) {
 	buf := &bytes.Buffer{}
-	enc := codec.NewEncoder(buf, &codec.MsgpackHandle{})
-	dec := codec.NewDecoder(buf, &codec.MsgpackHandle{})
+	enc := gocodec.NewEncoder(buf, &gocodec.MsgpackHandle{})
+	dec := gocodec.NewDecoder(buf, &gocodec.MsgpackHandle{})
 
 	var out int
 	for i := 0; i < b.N; i++ {
@@ -870,8 +838,8 @@ func BenchmarkStringSlicePtr(b *testing.B) {
 
 type benchmarkStruct struct {
 	Name      string
-	Colors    []string
 	Age       int
+	Colors    []string
 	Data      []byte
 	CreatedAt time.Time
 	UpdatedAt time.Time
@@ -879,8 +847,8 @@ type benchmarkStruct struct {
 
 type benchmarkStruct2 struct {
 	Name      string
-	Colors    []string
 	Age       int
+	Colors    []string
 	Data      []byte
 	CreatedAt time.Time
 	UpdatedAt time.Time
@@ -967,29 +935,29 @@ func BenchmarkStructManual(b *testing.B) {
 	}
 }
 
-func BenchmarkStructMsgpack2(b *testing.B) {
+func BenchmarkStructUgorjiGoMsgpack(b *testing.B) {
 	in := structForBenchmark()
 	out := &benchmarkStruct{}
 	for i := 0; i < b.N; i++ {
-		buf, err := msgpack2.Marshal(in)
+		buf, err := gomsgpack.Marshal(in)
 		if err != nil {
 			b.Fatal(err)
 		}
 
-		err = msgpack2.Unmarshal(buf, out, nil)
+		err = gomsgpack.Unmarshal(buf, out, nil)
 		if err != nil {
 			b.Fatal(err)
 		}
 	}
 }
 
-func BenchmarkStructMsgpack3(b *testing.B) {
+func BenchmarkStructUgorjiGoCodec(b *testing.B) {
 	in := structForBenchmark()
 	out := &benchmarkStruct{}
 	for i := 0; i < b.N; i++ {
 		buf := &bytes.Buffer{}
-		enc := codec.NewEncoder(buf, &codec.MsgpackHandle{})
-		dec := codec.NewDecoder(buf, &codec.MsgpackHandle{})
+		enc := gocodec.NewEncoder(buf, &gocodec.MsgpackHandle{})
+		dec := gocodec.NewDecoder(buf, &gocodec.MsgpackHandle{})
 
 		if err := enc.Encode(in); err != nil {
 			b.Fatal(err)
@@ -1034,6 +1002,47 @@ func BenchmarkStructGOB(b *testing.B) {
 	}
 }
 
+func BenchmarkStructDecode(b *testing.B) {
+	in := structForBenchmark()
+	buf, err := msgpack.Marshal(in)
+	if err != nil {
+		b.Fatal(err)
+	}
+	out := &benchmarkStruct{}
+
+	b.ResetTimer()
+
+	for i := 0; i < b.N; i++ {
+		err = msgpack.Unmarshal(buf, out)
+		if err != nil {
+			b.Fatal(err)
+		}
+	}
+}
+
+type benchmarkSubStruct struct {
+	Name string
+	Age  int
+}
+
+func BenchmarkStructDecodePartially(b *testing.B) {
+	in := structForBenchmark()
+	buf, err := msgpack.Marshal(in)
+	if err != nil {
+		b.Fatal(err)
+	}
+	out := &benchmarkSubStruct{}
+
+	b.ResetTimer()
+
+	for i := 0; i < b.N; i++ {
+		err = msgpack.Unmarshal(buf, out)
+		if err != nil {
+			b.Fatal(err)
+		}
+	}
+}
+
 func BenchmarkCSV(b *testing.B) {
 	for i := 0; i < b.N; i++ {
 		record := []string{"1", "hello", "world"}
